@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
+# coding: utf-8
 from django.core.exceptions import ImproperlyConfigured
-from django.views.generic.base import TemplateResponseMixin
 from django.views.generic.list import ListView
 from .config import RequestConfig
 
@@ -10,15 +9,17 @@ class SingleTableMixin(object):
     Adds a Table object to the context. Typically used with
     ``TemplateResponseMixin``.
 
-    :param table_class: table class
-    :type table_class: subclass of ``django_tables2.Table``
-
-    :param table_data: data used to populate the table
-    :type table_data: any compatible data source
-
+    :param        table_class: table class
+    :type         table_class: subclass of ``django_tables2.Table``
+    :param         table_data: data used to populate the table
+    :type          table_data: any compatible data source
     :param context_table_name: name of the table's template variable (default:
-        "table")
-    :type context_table_name: ``string``
+                               "table")
+    :type  context_table_name: ``string``
+    :param   table_pagination: controls table pagination. If a dict, passed as
+                               the ``paginate`` keyword argument to
+                               ``RequestConfig()``. As such, any *non-False*
+                               value enables pagination.
 
     This mixin plays nice with the Django's ``MultipleObjectMixin`` by using
     ``get_queryset()`` as a fallback for the table data source.
@@ -26,15 +27,20 @@ class SingleTableMixin(object):
     table_class = None
     table_data = None
     context_table_name = None
+    table_pagination = None
 
     def get_table(self):
         """
         Return a table object to use. The table has automatic support for
         sorting and pagination.
         """
+        options = {}
         table_class = self.get_table_class()
         table = table_class(self.get_table_data())
-        RequestConfig(self.request).configure(table)
+        paginate = self.get_table_pagination()  # pylint: disable=E1102
+        if paginate is not None:
+            options['paginate'] = paginate
+        RequestConfig(self.request, **options).configure(table)
         return table
 
     def get_table_class(self):
@@ -65,6 +71,13 @@ class SingleTableMixin(object):
                                    u"%(cls)s.table_data"
                                    % {"cls": type(self).__name__})
 
+    def get_table_pagination(self):
+        """
+        Returns pagination options: True for standard pagination (default),
+        False for no pagination, and a dictionary for custom pagination.
+        """
+        return self.table_pagination
+
     def get_context_data(self, **kwargs):
         """
         Overriden version of ``TemplateResponseMixin`` to inject the table into
